#include "App.h"

#include "AMI.h"
#include "events.h"

#include <SDL2/SDL.h>
#include <SDL2/SDL_events.h>
#include <SDL2/SDL_image.h>
#include <SDL2/SDL_keycode.h>
#include <fmt/format.h>

#include <queue>
#include <stdexcept>

App::App() : view{AppView::AMI} { init_sdl(); }

App::~App() {
  // Destroy window
  SDL_DestroyRenderer(renderer);
  renderer = nullptr;
  SDL_DestroyWindow(window);
  window = nullptr;
}

void App::init_sdl() {
  window = SDL_CreateWindow("FT22 STW Display", SDL_WINDOWPOS_UNDEFINED,
                            SDL_WINDOWPOS_UNDEFINED, SCREEN_WIDTH,
                            SCREEN_HEIGHT, SDL_WINDOW_SHOWN);
  if (window == nullptr) {
    throw std::runtime_error(
        fmt::format("Couldn't create window: {}", SDL_GetError()));
  }
  renderer = SDL_CreateRenderer(window, -1, SDL_RENDERER_ACCELERATED);
  if (renderer == nullptr) {
    throw std::runtime_error(
        fmt::format("Couldn't create renderer: {}", SDL_GetError()));
  }
}

int App::run() {
  ami = std::make_unique<AMI>(renderer);

  running = true;

  while (running) {
    handle_events();
    render();
    SDL_Delay(1);
  }

  return 0;
}

void App::handle_events() {
  std::queue<Event> events;
  SDL_Event e;
  while (SDL_PollEvent(&e) != 0) {
    if (e.type == SDL_QUIT) {
      running = false;
    } else if (e.type == SDL_KEYUP) {
      switch (e.key.keysym.sym) {
      case SDLK_DOWN:
      case SDLK_RIGHT:
        events.push(Event::Next);
        break;
      case SDLK_UP:
      case SDLK_LEFT:
        events.push(Event::Prev);
        break;
      case SDLK_RETURN:
      case SDLK_KP_ENTER:
        events.push(Event::Confirm);
        break;
        // We can just ignore other keypresses, so no need for a default clause
      }
    }
  }

  switch (view) {
  case AppView::AMI:
    ami->handle_events(events);
    break;
  default:
    throw std::runtime_error(fmt::format("Unknown view: {}", (int)view));
  }
}

void App::render() {
  switch (view) {
  case AppView::AMI:
    ami->draw();
    break;
  default:
    throw std::runtime_error(fmt::format("Unknown view: {}", (int)view));
  }
  SDL_RenderPresent(renderer);
}

SDLManager::SDLManager() {
  if (SDL_Init(SDL_INIT_VIDEO) < 0) {
    throw std::runtime_error(
        fmt::format("Couldn't initialize SDL: {}", SDL_GetError()));
  }
  if (TTF_Init() == -1) {
    throw std::runtime_error(
        fmt::format("Couldn't initialize SDL_ttf: {}", TTF_GetError()));
  }
  int img_flags = IMG_INIT_PNG;
  if (!(IMG_Init(img_flags) & img_flags)) {
    throw std::runtime_error(
        fmt::format("Couldn't initialitze SDL_image: {}", IMG_GetError()));
  }
}

SDLManager::~SDLManager() {
  IMG_Quit();
  TTF_Quit();
  SDL_Quit();
}